<?php
//extend woocommerce class
if(!function_exists('woocommerce_payzah_init')){
	function woocommerce_payzah_init(){
		class WC_Payzah extends WC_Payment_Gateway {
			//constructor for the gateway.
			public function __construct() {
				$this->id                 = 'payzah';
				$this->icon               = payzah_image_dir . 'accept_payment.png';
				$this->has_fields         = false;
				$this->method_title       = __( 'Payzah Payment Gateway', 'payzah' );
				$this->method_description = __( 'Pay with payzah that supports KNET & Credit Cards Transactions (VISA & MasterCard)', 'payzah' );
				$this->supports           = array( 'products', 'refunds' );
				
				// Load the settings.
				$this->init_form_fields();
				$this->init_settings();

				$this->options =  array();
				// Define "payment type" radio buttons options field
				if ($this->settings['payzah_knet_option'] == "yes" && $this->settings['payzah_cc_option'] == "yes"  ) {
					$this->options = array(
						'1' => __( 'KNET', 'payzah' ),
						'2' => __( 'Credit Card', 'payzah' ),
					);
				}
				if ($this->settings['payzah_knet_option'] == "yes" && $this->settings['payzah_cc_option'] == "no" ) {
					$this->options = array(
						'1' => __( 'KNET', 'payzah' )
					);
				}
				if ($this->settings['payzah_cc_option'] == "yes" && $this->settings['payzah_knet_option'] == "no") {
					$this->options = array(
						'2' => __( 'Credit Card', 'payzah' )
					);
				}

				if(!empty($this->settings['title'])){
					$title = $this->settings['title'];
				}else{
					$title = __( 'Payzah', 'payzah' );
				}
				$this->title 	= $title;
				$this->description 		= $this->settings['description'];
				$this->private_key 			= $this->settings['private_key'];
				if ( $this->settings['testmode'] == "yes" ) {
					$this->apiurl 			= 'https://development.payzah.net/ws/paymentgateway/index';
					$this->refundapiurl     = 'https://development.payzah.net/ws/paymentgateway/refund';
				} else {
					$this->apiurl 			= 'https://payzah.net/production770/ws/paymentgateway/index';
					$this->refundapiurl     = 'https://payzah.net/production770/ws/paymentgateway/refund';
				}


				if ( version_compare( WOOCOMMERCE_VERSION, '2.0.0', '>=' ) ) {
				/* 2.0.0 */
				add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( &$this, 'process_admin_options' ) );
				} else {
				/* 1.6.6 */
				add_action( 'woocommerce_update_options_payment_gateways', array( &$this, 'process_admin_options' ) );
				}

			}

			//fields for bacend form
			function init_form_fields(){
				$this->form_fields = array(
					'enabled' => array(
						'title' 		=> __('Enable/Disable', 'payzah'),
						'type' 			=> 'checkbox',
						'label' 		=> __('Enable Payzah Payment Module.', 'payzah'),
						'default' 		=> 'no',
						'description' 	=> 'Show in the Payment List as a payment option'
					),
	      			'title' => array(
						'title' 		=> __('Title:', 'payzah'),
						'type'			=> 'text',
						'default' 		=> __('Payzah', 'payzah'),
						'description' 	=> __('This controls the title which the user sees during checkout.', 'payzah'),
						'desc_tip' 		=> true
					),
	      			'description' => array(
						'title' 		=> __('Description:', 'payzah'),
						'type' 			=> 'textarea',
						'default' 		=> __('pay via Payzah; you can pay securely with your debit or credit card.', 'payzah'),
						'description' 	=> __('This controls the description which the user sees during checkout.', 'payzah'),
						'desc_tip' 		=> true
					),
	      			'private_key' => array(
						'title'       => __('Private key', 'payzah'),
						'type'        => 'text',
						'value'       => '',
						'description' => __( 'Get your Private key from Payzah.','woocommerce' ),
						'default'     => '',
						'desc_tip'    =>true,
						'required'    =>true),
	      			'testmode' => array(
						'title' 		=> __('TEST Mode', 'payzah'),
						'type' 			=> 'checkbox',
						'label' 		=> __('Enable Payzah TEST Transactions.', 'payzah'),
						'default' 		=> 'no',
						'description' 	=> __('Tick to run TEST Transaction on the Payzah platform'),
						'desc_tip' 		=> true
	                ),
	                'payzah_knet_option' => array(
						'title' 		=> __('Enable Knet', 'payzah'),
						'type' 			=> 'checkbox',
						'label' 		=> __('Enable Knet Option.', 'payzah'),
						'default' 		=> 'yes',
						'description' 	=> __('Tick to enable knet option.'),
						'desc_tip' 		=> true
	                ),
	                'payzah_cc_option' => array(
						'title' 		=> __('Enable Credit Card', 'payzah'),
						'type' 			=> 'checkbox',
						'label' 		=> __('Enable Credit Card Option.', 'payzah'),
						'default' 		=> 'yes',
						'description' 	=> __('Tick to enable credit card option.'),
						'desc_tip' 		=> true
	                )

				);
			}

			//output the "payment type" radio buttons fields in checkout.
			public function payment_fields(){
				//currency should be  Kuwaiti currency
				if(get_woocommerce_currency() == 'KWD'){	
		            if ( $description = $this->get_description() ) {
		                echo wpautop( wptexturize( $description ) );
		            }
		            echo '<style>#payzah_transaction_type_field label.radio { display:inline-block; margin:0 .8em 0 .4em}</style>';
		            $option_keys = array_keys($this->options);
		            woocommerce_form_field( 'payzah_transaction_type', array(
		                'type'          => 'radio',
		                'class'         => array('payzah_transaction_type form-row-wide'),
		                'label'         => __('Payment Information', 'payzah'),
		                'options'       => $this->options,
		                'required'      => true,
		            ), reset( $option_keys ) );
		        }else{
		        	echo 'Payzah only support kuwaiti currency, default payment type is KNET';
		        }
       		}

       		//when payment will process from front end
			public function process_payment( $order_id ) {
				$url = $this->apiurl;
				if(!empty($this->private_key)){
					$private_key = base64_encode($this->private_key);
					$order = new WC_Order( $order_id );
					$order_data = $order->get_data();
					$order_billing_full_name = $order_data['billing']['first_name'].' '.$order_data['billing']['last_name'];
					$order_billing_postcode = $order_data['billing']['postcode'];
					$order_billing_phone = $order_data['billing']['phone'];
					$order_billing_city = $order_data['billing']['city'];
					$lang=get_bloginfo("language");
					$lang = 'ENG';
					if($selected_lang == 'ar'){
						$lang = 'ARA';
					}
					if ( $_POST['payment_method'] === "payzah" && isset($_POST['payzah_transaction_type']) ){
						$payment_type = sanitize_text_field($_POST['payzah_transaction_type']);
					}else{
						$payment_type = 1;
					}
					$data_string = array(
					   "trackid" => $order_id,
					   "amount"=> $order->order_total,
					   "success_url" => site_url('/?payzah_callback=payzah_response_handle'),
					   "error_url" => site_url('/?payzah_callback=payzah_response_handle'),
					   "language" => $lang, 
					   "currency" =>  "414",
					   "udf1" => $order_billing_full_name,
					   "udf2" => $order_billing_phone,
					   "udf3" => $order_billing_postcode,
					   "udf4" => $order_billing_city,
					   "payment_type" => $payment_type
					);
					$data_string = json_encode($data_string);
					$headers = array( 
						'Authorization' => $private_key,
						'Content-type' => 'application/json'
					);
					$post_data = array(
						'method' => 'POST',
						'timeout' => 30,
						'headers' => $headers,
						'body' => $data_string,
					);
					$response = wp_remote_post($url, $post_data);
					$api_response = json_decode($response['body']);
					if($api_response->status == 1){
						$api_redirect_url = $api_response->data->PaymentUrl;
						$spi_id = $api_response->data->PaymentID;
						$redirect_url_for_payment = $api_redirect_url.'/?PaymentID='.$spi_id;
						return array(
							'result' 	=> 'success',
							'redirect'	=> $redirect_url_for_payment
						);
					}
					if($api_response->status != 1){
						wc_add_notice( $api_response->message , 'error' );
					}

				}else{
					wc_add_notice(  "Technical error has been occured in Pazah, please contact to site owner", 'error' );
				}
			}

			//refund process
			public function process_refund($order_id, $amount = null, $reason = ''){
				$refundapiurl = $this->refundapiurl;
				$private_key = base64_encode($this->private_key);
				$payzah_response = unserialize(get_post_meta($order_id,'_payzah_response',true));
				$order = new WC_Order( $order_id );
				$order_total = $order->get_total();
				$refund_amount = sanitize_text_field($_POST['refund_amount']);
				if($order_total == $refund_amount){
					$refund_type = 1;
				}else{
					$refund_type = 2;
				}
				if(!empty($_POST['refund_reason'])){
					$refund_reason = sanitize_text_field($_POST['refund_reason']);
				}else{
					$refund_reason = '';
				}
				$refund_data = array(
				   'trackid' => $order_id,
				   'refrence_code' => $payzah_response['payzahRefrenceCode'],
				   'amount'=> $refund_amount,
				   'refund_type' => $refund_type, 
				   'message' =>  $refund_reason
				);
				$refund_data = json_encode($refund_data);
				$headers = array( 
					'Authorization' => $private_key,
					'Content-type' => 'application/json'
				);
				$post_data = array(
					'method' => 'POST',
					'timeout' => 30,
					'headers' => $headers,
					'body' => $refund_data,
				);
				$refund_response = wp_remote_post($refundapiurl, $post_data);
				$refund_api_response = json_decode($refund_response['body']);
				if($refund_api_response->status == 1){
					update_post_meta($order_id,'_payzah_refund_response', sanitize_text_field(serialize($_POST)),true);
					wp_update_post(array('ID' => $order_id,'post_status' => 'wc-refunded'));
					$order->add_order_note('Refunded Successfully.');
				}
				if($refund_api_response->status != 1){
					return new WP_Error( 'error', __($refund_api_response->message, 'woocommerce' ) );
				}
				return true;
			}
		}
	}
}